function [yi, u] = barylag(x, y, xi)

% BARYLAG 1-D barycentric lagrange interpolation
%    BARYLAG(X,Y,XI) interpolates to find YI, the values of the
%    underlying function Y at the points in the array XI, using the
%    second form (or true form) of the barycentric interpolation
%    formula.  X and Y must be vectors of length N.
%
%    [YI,U] = BARYLAG() also returns the barycentric interpolation
%    weights U.
%
%    Ref:
%    Barycentric Lagrange Interpolation
%    Jean-Paul Berrut and Lloyd N. Trefethen
%    SIAM Review, Vol. 46, No. 3
%    pgs. 501-517

% Joe Henning - Fall 2011

n = length(x);

if n ~= length(y)
   fprintf('??? Bad input to barylag ==> X and Y must be of the same length\n');
   yi = [];
   u = [];
   return;
end

for i = 1:length(xi)
   % Find the right place in the table by means of a bisection.
   klo = 1;
   khi = n;
   while (khi-klo > 1)
      k = fix((khi+klo)/2.0);
      if (x(k) > xi(i))
         khi = k;
      else
         klo = k;
      end
   end
   
   h = x(khi) - x(klo);
   if (h == 0.0)
      fprintf('??? Bad input to barylag ==> X values must be distinct\n');
      yi(i) = NaN;
      continue;
   end

   isiny = 0;
   for k = 1:n
      if (xi(i) == x(k))
         yi(i) = y(k);
         isiny = 1;
         break
      end
   end

   if (isiny)
      continue
   end

   % Evaluate lagrange polynomial
   num = 0;
   den = 0;
   for k = 1:n
      u(k,1) = 1;
      for m = 1:n
         if (k ~= m)
            u(k,1) = u(k,1)/(x(k)-x(m));
         end
      end
      num = num + u(k,1)*y(k)/(xi(i)-x(k));
      den = den + u(k,1)/(xi(i)-x(k));
   end
   yi(i) = num/den;
end